setGeneric ("ndisease",
  function(object){standardGeneric("ndisease")}
)
setGeneric ("ngene",
  function(object){standardGeneric("ngene")}
)
setGeneric ("extract",
  function(object){standardGeneric("extract")}
)
setGeneric ("extractTable",
  function(object){standardGeneric("extractTable")}
)
setGeneric ("plotN",
  function(x,...){standardGeneric("plotN")}
)
